from jaloninterface import IJalonContent
from jaloninterface import IJalonFolder
from jaloninterface import IJalonCours
from jaloninterface import IJalonActivity
from jaloninterface import IJalonCoursWims
from jaloninterface import IJalonBoiteDepot
from jaloninterface import IJalonRessourceExterne
from jaloninterface import IJalonTermeGlossaire
from jaloninterface import IJalonConnect
from jaloninterface import IJalonExerciceWims
from jaloninterface import IJalonFile
from jaloninterface import IJalonForum
from jaloninterface import IJalonAnnonce