from my_space_view import MySpaceView
from my_files_view import MyFilesView
from my_adobe_connect_recording_view import MyAdobeConnectRecordingView
from my_wims_exercices_view import MyWimsExercicesView
from my_external_resources_view import MyExternalResourcesView
from my_glossary_terms_view import MyGlossaryTermsView
from my_pod_videos_view import MyPodVideosView
from my_courses_view import MyCoursesView
from course_view import CourseView
from deposit_box_view import DepositBoxView
from wims_activity_view import WimsActivityView
from deposit_box_criteria_view import DepositBoxCriteriaView
from deposit_box_evaluation_view import DepositBoxEvaluationView
from course_statistics_view import CourseStatisticsView
from my_students_view import MyStudentsView
