"""Common configuration constants."""
PROJECTNAME = 'jalon.content'

ADD_PERMISSIONS = {
    # -*- extra stuff goes here -*-
    'JalonFolder'           : 'Add Jalon Folder',
    'JalonCours'            : 'Add Jalon Cours',
    'JalonActivity'         : 'Add Jalon Activity',
    'JalonCoursWims'        : 'Add Jalon Cours auto-evaluation, examen',
    'JalonBoiteDepot'       : 'Add Jalon BoiteDepots',
    'JalonRessourceExterne' : 'Add Jalon RessourceExterne',
    'JalonTermeGlossaire'   : 'Add Jalon TermeGlossaire',
    'JalonConnect'          : 'Add Jalon Connect',
    'JalonExerciceWims'     : 'Add Jalon ExerciceWims',
    'JalonFile'             : 'Add Jalon File',
    'JalonForum'            : 'Add Jalon Forum',
    'JalonAnnonce'          : 'Add Jalon Annonce',
}
