## Controller Python Script "order_course_map_item_script"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=
##

context.orderCourseMapItems(context.REQUEST.form["plan"])
