import jalonfolder
import jaloncours
import jalonboitedepot
import jalonressourceexterne
import jalontermeglossaire
import jalonconnect
import jalon_utils
import jalonexercicewims
import jaloncourswims
import jalonfile
import jalonforum
import jalonannonce
import jalon_encode
import jalon_properties
import jalon_activity