# -*- coding: utf-8 -*-

from zope.interface import Interface


class IJalonContent(Interface):
    ''' Interface for the JalonContent type
    '''


class IJalonFolder(Interface):
    ''' Interface for the JalonFolder type
    '''


class IJalonCours(Interface):
    ''' Interface for the JalonCours type
    '''


class IJalonCoursWims(Interface):
    ''' Interface for the JalonCoursWims type
    '''


class IJalonActivity(Interface):
    ''' Interface for the JalonActivity type
    '''


class IJalonBoiteDepot(Interface):
    ''' Interface for the JalonBoiteDepot type
    '''


class IJalonRessourceExterne(Interface):
    ''' Interface for the JalonRessourceExtern type
    '''


class IJalonTermeGlossaire(Interface):
    ''' Interface for the JalonTermeGlossaire type
    '''


class IJalonConnect(Interface):
    ''' Interface for the JalonConnect type
    '''


class IJalonExerciceWims(Interface):
    ''' Interface for the JalonExerciceWims type
    '''


class IJalonFile(Interface):
    ''' Interface for the JalonFile type
    '''


class IJalonForum(Interface):
    ''' Interface for the JalonForum type
    '''


class IJalonAnnonce(Interface):
    ''' Interface for the JalonAnnonce type
    '''
